@@ -22,7 +22,7 @@ if(is.numeric(alpha)) if(!(alpha<1&&alpha>0) ) stop(" alpha must be  numeric bet
 
 model=fit$model
 
-print("Parametric bootstrap can be time consuming ...")
+cat("Parametric bootstrap can be time consuming ...\n")
 
 if(fit$missp)  ### misspecification
  {if(fit$model=="StudentT")     model="Gaussian_misp_StudentT"
@@ -41,64 +41,44 @@ if(sum(fit$X[1:dimat]==1)==dimat&&!dim(fit$X)[2]>1) fit$X=NULL
 k=1;res=NULL
 
 
-  pb <- txtProgressBar(min = 0, max = K, style = 3)
+
   coords=cbind(fit$coordx,fit$coordy)
   N=nrow(coords)
   pp=NULL
+######## simulation ##########################################
+if(is.null(fit$copula)){     ### non copula models
+   cat("Performing simulation....\n")
+    if(method=="cholesky")
+    { 
+
+    
+      data_sim = GeoSim(coordx=coords,coordt=fit$coordt,coordx_dyn=fit$coordx_dyn, anisopars=fit$anisopars,
+      corrmodel=fit$corrmodel,model=fit$model,param=append(fit$param,fit$fixed),
+      GPU=GPU,  local=local,sparse=sparse,grid=fit$grid, X=fit$X,n=fit$n,method=method,
+      distance=fit$distance,radius=fit$radius,nrep=K)}
+
+   if(method=="TB"||method=="Vecchia"||method=="CE")   
+     { data_sim = GeoSimapprox(coordx=coords,coordt=fit$coordt,coordx_dyn=fit$coordx_dyn, anisopars=fit$anisopars,
+      corrmodel=fit$corrmodel,model=fit$model,param=append(fit$param,fit$fixed),
+      GPU=GPU,  local=local,grid=fit$grid,X=fit$X,n=fit$n,method=method,
+      M=M, L=L,distance=fit$distance,radius=fit$radius,nrep=K)}
+}
+else{    ### copula models
+       cat("Performing simulation....\n")
+        if(method=="cholesky")
+     { data_sim = GeoSimCopula(coordx=coords,coordt=fit$coordt,coordx_dyn=fit$coordx_dyn, anisopars=fit$anisopars,
+       corrmodel=fit$corrmodel,model=fit$model,copula=fit$copula,param=append(fit$param,fit$fixed),
+       GPU=GPU,  local=local,sparse=sparse,grid=fit$grid,X=fit$X,n=fit$n,method=method,
+       distance=fit$distance,radius=fit$radius,nrep=K)}
+}
+###############################################################
+  cat("Performing estimation...\n")
+  pb <- txtProgressBar(min = 0, max = K, style = 3)
+
 while(k<=K){
     Sys.sleep(0.1)
-if(is.null(fit$copula)){
-    if(method=="cholesky")
-    {
-data_sim = GeoSim(coordx=coords,coordt=fit$coordt,
-     coordx_dyn=fit$coordx_dyn, anisopars=fit$anisopars,
-     corrmodel=fit$corrmodel,model=fit$model,
-	 #param=as.list(c(fit$param,fit$fixed)),
-      param=append(fit$param,fit$fixed),
-	 GPU=GPU,  local=local,sparse=sparse,#grid=fit$grid, 
-     X=fit$X,n=fit$n,method=method,
-	 distance=fit$distance,radius=fit$radius)
-    }
-     if(method=="TB"||method=="Vecchia"||method=="CE")   
-     {
-  
- data_sim = GeoSimapprox(coordx=coords,coordt=fit$coordt,
-     coordx_dyn=fit$coordx_dyn, anisopars=fit$anisopars,
-     corrmodel=fit$corrmodel,model=fit$model,
-      param=append(fit$param,fit$fixed),
-     GPU=GPU,  local=local,#grid=fit$grid, 
-     X=fit$X,n=fit$n,method=method,
-     M=M, L=L,
-     distance=fit$distance,radius=fit$radius)
 
-    }
-}
-else{
-    if(method=="cholesky")
-    {
- data_sim = GeoSimCopula(coordx=coords,coordt=fit$coordt,
-     coordx_dyn=fit$coordx_dyn, anisopars=fit$anisopars,
-     corrmodel=fit$corrmodel,model=fit$model,
-     copula=fit$copula,
-     param=append(fit$param,fit$fixed),
-     GPU=GPU,  local=local,sparse=sparse,#grid=fit$grid, 
-     X=fit$X,n=fit$n,method=method,
-     distance=fit$distance,radius=fit$radius)
-     }
-}
-
-
-
-
-#print(append(fit$param,fit$fixed))
-#if(method=="Vecchia"||method=="TB") {
- #               data_sim = GeoSimapprox(coordx=coords,coordt=fit$coordt, coordx_dyn=fit$coordx_dyn, corrmodel=fit$corrmodel,model=fit$model, 
-  #              param=append(fit$param,fit$fixed),method=method,M=M,L=L,GPU=GPU,  local=local,#grid=fit$grid, 
-   #             X=fit$X,n=fit$n,distance=fit$distance,radius=fit$radius)
-    #        }
-
-
-res_est=GeoFit( data=data_sim$data, start=fit$param,fixed=fit$fixed,#start=as.list(fit$param),fixed=as.list(fit$fixed),
+res_est=GeoFit( data=data_sim$data[[k]], start=fit$param,fixed=fit$fixed,
    coordx=coords, coordt=fit$coordt, coordx_dyn=fit$coordx_dyn,
    copula=fit$copula,sensitivity=FALSE,anisopars=fit$anisopars,est.aniso=fit$est.aniso,
    lower=lower,upper=upper,memdist=TRUE,neighb=fit$neighb,
@@ -108,9 +88,6 @@ res_est=GeoFit( data=data_sim$data, start=fit$param,fixed=fit$fixed,#start=as.li
    X=fit$X, distance=fit$distance, radius=fit$radius)
 
 
-
-
-
 if(res_est$convergence=='Successful'&&res_est$logCompLik<1.0e8) 
  {
  res=rbind(res,unlist(res_est$param))
