@@ -1093,258 +1093,3 @@ if(!ISNAN(data1[i])&&!ISNAN(data2[i]) ){
 /******************************************************************************************/
 
 
-
-/******************************************************************************************/
-/******************************************************************************************/
-/******************************************************************************************/
-/********************* SPATIAL Gaussian COPULA *****************************************************/
-/******************************************************************************************/
-/******************************************************************************************/
-/******************************************************************************************/
-/******************************************************************************************/
-
-// Composite marginal (pariwise) log-likelihood for the spatial Gaussian model:
-void Comp_Cond_GaussGCop2mem_aniso(int *cormod, double *coord1, double *coord2,double *data1,double *data2,int *NN,
- double *par, int *weigthed, double *res,double *mean1,double *mean2,
- double *nuis, int *local,int *GPU)
-{
-    int type_cop=1; int model=1; int cond=1;
-    /*############*/
-    int i=0;double lag=0.0;
-    double  weights=1.0,sill,nugget,corr,bl;
-    sill=nuis[1];nugget=nuis[0];
-    if(sill<0 || nugget<0||nugget>1){*res=LOW; return;}
-
-
-    for(i=0;i<*npairs;i++){
-if(!ISNAN(data1[i])&&!ISNAN(data2[i]) ){
-      lag= hypot(coord1[2*i]-coord2[2*i],coord1[2*i+1]-coord2[2*i+1]); 
-                      corr=CorFct(cormod,lag,0,par,0,0);
-                       if(*weigthed) weights=CorFunBohman(lag,maxdist[0]);
-           bl=biv_cop(corr,type_cop,cond,data1[i],data2[i],mean1[i],mean2[i],nuis,model,NN[0]);
-                      *res+= bl*weights;
-                    }}
-    if(!R_FINITE(*res))  *res = LOW;
-    return;
-}
-
-
-
-
-void Comp_Cond_BetaGCop2mem_aniso(int *cormod, double *coord1, double *coord2,double *data1,double *data2,int *NN,
- double *par, int *weigthed, double *res,double *mean1,double *mean2,
- double *nuis, int *local,int *GPU)
-{
-    int type_cop=1; int model=28; int cond=1;
-    /*############*/
-    int i=0;double lag=0.0;
-    double  weights=1.0,sill,nugget,corr,bl;
-    sill=nuis[1];nugget=nuis[0];
-    if(sill<0 || nugget<0||nugget>1){*res=LOW; return;}
-
-    for(i=0;i<*npairs;i++){
-if(!ISNAN(data1[i])&&!ISNAN(data2[i]) ){
-      lag= hypot(coord1[2*i]-coord2[2*i],coord1[2*i+1]-coord2[2*i+1]); 
-                      corr=CorFct(cormod,lag,0,par,0,0);
-                       if(*weigthed) weights=CorFunBohman(lag,maxdist[0]);
-           bl=biv_cop(corr,type_cop,cond,data1[i],data2[i],mean1[i],mean2[i],nuis,model,NN[0]);
-                        *res+= bl*weights;
-                    }}
-    if(!R_FINITE(*res))  *res = LOW;
-    return;
-}
-
-void Comp_Cond_Beta2GCop2mem_aniso(int *cormod, double *coord1, double *coord2,double *data1,double *data2,int *NN,
- double *par, int *weigthed, double *res,double *mean1,double *mean2,
- double *nuis, int *local,int *GPU)
-{
-    int type_cop=1; int model=50; int cond=1;
-    /*############*/
-    int i=0;double lag=0.0;
-    double  weights=1.0,sill,nugget,corr,bl;
-    sill=nuis[1];nugget=nuis[0];
-    if(sill<0 || nugget<0||nugget>1){*res=LOW; return;}
-
-    for(i=0;i<*npairs;i++){
-if(!ISNAN(data1[i])&&!ISNAN(data2[i]) ){
-      lag= hypot(coord1[2*i]-coord2[2*i],coord1[2*i+1]-coord2[2*i+1]); 
-                      corr=CorFct(cormod,lag,0,par,0,0);
-                       if(*weigthed) weights=CorFunBohman(lag,maxdist[0]);
-           bl=biv_cop(corr,type_cop,cond,data1[i],data2[i],mean1[i],mean2[i],nuis,model,NN[0]);
-                       *res+= bl*weights;
-                    }}
-    if(!R_FINITE(*res))  *res = LOW;
-    return;
-}
-
-void Comp_Cond_KumaraswamyGCop2mem_aniso(int *cormod, double *coord1, double *coord2,double *data1,double *data2,int *NN,
- double *par, int *weigthed, double *res,double *mean1,double *mean2,
- double *nuis, int *local,int *GPU)
-{
-    int type_cop=1; int model=33; int cond=1;
-    /*############*/
-    int i=0;double lag=0.0;
-    double  weights=1.0,sill,nugget,corr,bl;
-    sill=nuis[1];nugget=nuis[0];
-    if(sill<0 || nugget<0||nugget>1){*res=LOW; return;}
-
-    for(i=0;i<*npairs;i++){
-if(!ISNAN(data1[i])&&!ISNAN(data2[i]) ){
-      lag= hypot(coord1[2*i]-coord2[2*i],coord1[2*i+1]-coord2[2*i+1]); 
-                      corr=CorFct(cormod,lag,0,par,0,0);
-                       if(*weigthed) weights=CorFunBohman(lag,maxdist[0]);
-           bl=biv_cop(corr,type_cop,cond,data1[i],data2[i],mean1[i],mean2[i],nuis,model,NN[0]);
-                        *res+= bl*weights;
-                    }}
-    if(!R_FINITE(*res))  *res = LOW;
-    return;
-}
-void Comp_Cond_Kumaraswamy2GCop2mem_aniso(int *cormod, double *coord1, double *coord2,double *data1,double *data2,int *NN,
- double *par, int *weigthed, double *res,double *mean1,double *mean2,
- double *nuis, int *local,int *GPU)
-{
-    int type_cop=1; int model=42; int cond=1;
-    /*############*/
-    int i=0;double lag=0.0;
-    double  weights=1.0,sill,nugget,corr,bl;
-    sill=nuis[1];nugget=nuis[0];
-    if(sill<0 || nugget<0||nugget>1){*res=LOW; return;}
-
-    for(i=0;i<*npairs;i++){
-if(!ISNAN(data1[i])&&!ISNAN(data2[i]) ){
-      lag= hypot(coord1[2*i]-coord2[2*i],coord1[2*i+1]-coord2[2*i+1]); 
-                      corr=CorFct(cormod,lag,0,par,0,0);
-                       if(*weigthed) weights=CorFunBohman(lag,maxdist[0]);
-           bl=biv_cop(corr,type_cop,cond,data1[i],data2[i],mean1[i],mean2[i],nuis,model,NN[0]);
-                      *res+= bl*weights;
-                    }}
-    if(!R_FINITE(*res))  *res = LOW;
-    return;
-}
-
-
-
-/******************************************************************************************/
-/******************************************************************************************/
-/******************************************************************************************/
-/********************* SPATIAL Clayton COPULA *****************************************************/
-/******************************************************************************************/
-/******************************************************************************************/
-/******************************************************************************************/
-/******************************************************************************************/
-
-
-
-// Composite marginal (pariwise) log-likelihood for the spatial Gaussian model:
-void Comp_Cond_GaussCCop2mem_aniso(int *cormod, double *coord1, double *coord2,double *data1,double *data2,int *NN,
- double *par, int *weigthed, double *res,double *mean1,double *mean2,
- double *nuis, int *local,int *GPU)
-{
-    int type_cop=2; int model=1; int cond=1;
-    int i=0;double lag=0.0;
-    double  weights=1.0,sill,nugget,corr,bl;
-    sill=nuis[1];nugget=nuis[0];
-    if(sill<0 || nugget<0||nugget>1){*res=LOW; return;}
-    for(i=0;i<*npairs;i++){
-if(!ISNAN(data1[i])&&!ISNAN(data2[i]) ){
-  lag= hypot(coord1[2*i]-coord2[2*i],coord1[2*i+1]-coord2[2*i+1]); 
-                      corr=CorFct(cormod,lag,0,par,0,0);
-                       if(*weigthed) weights=CorFunBohman(lag,maxdist[0]);
-                       bl=biv_cop(corr,type_cop,cond,data1[i],data2[i],mean1[i],mean2[i],nuis,model,NN[0]);
-                        *res+= bl*weights;
-                    }}
-    if(!R_FINITE(*res))  *res = LOW;
-    return;
-}
-void Comp_Cond_BetaCCop2mem_aniso(int *cormod, double *coord1, double *coord2,double *data1,double *data2,int *NN,
- double *par, int *weigthed, double *res,double *mean1,double *mean2,
- double *nuis, int *local,int *GPU)
-{
-    int type_cop=2; int model=28; int cond=1;
-    /*############*/
-    int i=0;double lag=0.0;
-    double  weights=1.0,sill,nugget,corr,bl;
-    sill=nuis[1];nugget=nuis[0];
-    if(sill<0 || nugget<0||nugget>1){*res=LOW; return;}
-
-    for(i=0;i<*npairs;i++){
-if(!ISNAN(data1[i])&&!ISNAN(data2[i]) ){
-      lag= hypot(coord1[2*i]-coord2[2*i],coord1[2*i+1]-coord2[2*i+1]); 
-                      corr=CorFct(cormod,lag,0,par,0,0);
-                       if(*weigthed) weights=CorFunBohman(lag,maxdist[0]);
-           bl=biv_cop(corr,type_cop,cond,data1[i],data2[i],mean1[i],mean2[i],nuis,model,NN[0]);
-                      *res+= bl*weights;
-                    }}
-    if(!R_FINITE(*res))  *res = LOW;
-    return;
-}
-
-void Comp_Cond_Beta2CCop2mem_aniso(int *cormod, double *coord1, double *coord2,double *data1,double *data2,int *NN,
- double *par, int *weigthed, double *res,double *mean1,double *mean2,
- double *nuis, int *local,int *GPU)
-{
-    int type_cop=2; int model=50; int cond=1;
-    /*############*/
-    int i=0;double lag=0.0;
-    double  weights=1.0,sill,nugget,corr,bl;
-    sill=nuis[1];nugget=nuis[0];
-    if(sill<0 || nugget<0||nugget>1){*res=LOW; return;}
-
-    for(i=0;i<*npairs;i++){
-if(!ISNAN(data1[i])&&!ISNAN(data2[i]) ){
-      lag= hypot(coord1[2*i]-coord2[2*i],coord1[2*i+1]-coord2[2*i+1]); 
-                      corr=CorFct(cormod,lag,0,par,0,0);
-                       if(*weigthed) weights=CorFunBohman(lag,maxdist[0]);
-           bl=biv_cop(corr,type_cop,cond,data1[i],data2[i],mean1[i],mean2[i],nuis,model,NN[0]);
-                      *res+= bl*weights;
-                    }}
-    if(!R_FINITE(*res))  *res = LOW;
-    return;
-}
-
-
-
-void Comp_Cond_KumaraswamyCCop2mem_aniso(int *cormod, double *coord1, double *coord2,double *data1,double *data2,int *NN,
- double *par, int *weigthed, double *res,double *mean1,double *mean2,
- double *nuis, int *local,int *GPU)
-{
-    int type_cop=2; int model=33; int cond=1;
-    /*############*/
-    int i=0;double lag=0.0;
-    double  weights=1.0,sill,nugget,corr,bl;
-    sill=nuis[1];nugget=nuis[0];
-    if(sill<0 || nugget<0||nugget>1){*res=LOW; return;}
-
-    for(i=0;i<*npairs;i++){
-if(!ISNAN(data1[i])&&!ISNAN(data2[i]) ){
-      lag= hypot(coord1[2*i]-coord2[2*i],coord1[2*i+1]-coord2[2*i+1]); 
-                      corr=CorFct(cormod,lag,0,par,0,0);
-                       if(*weigthed) weights=CorFunBohman(lag,maxdist[0]);
-           bl=biv_cop(corr,type_cop,cond,data1[i],data2[i],mean1[i],mean2[i],nuis,model,NN[0]);
-                         *res+= bl*weights;
-                    }}
-    if(!R_FINITE(*res))  *res = LOW;
-    return;
-}
-void Comp_Cond_Kumaraswamy2CCop2mem_aniso(int *cormod, double *coord1, double *coord2,double *data1,double *data2,int *NN,
- double *par, int *weigthed, double *res,double *mean1,double *mean2,
- double *nuis, int *local,int *GPU)
-{
-    int type_cop=2; int model=42; int cond=1;
-    /*############*/
-    int i=0;double lag=0.0;
-    double  weights=1.0,sill,nugget,corr,bl;
-    sill=nuis[1];nugget=nuis[0];
-    if(sill<0 || nugget<0||nugget>1){*res=LOW; return;}
-
-    for(i=0;i<*npairs;i++){
-if(!ISNAN(data1[i])&&!ISNAN(data2[i]) ){
-      lag= hypot(coord1[2*i]-coord2[2*i],coord1[2*i+1]-coord2[2*i+1]); 
-                      corr=CorFct(cormod,lag,0,par,0,0);
-                       if(*weigthed) weights=CorFunBohman(lag,maxdist[0]);
-           bl=biv_cop(corr,type_cop,cond,data1[i],data2[i],mean1[i],mean2[i],nuis,model,NN[0]);
-                    *res+= bl*weights;
-                    }}
-    if(!R_FINITE(*res))  *res = LOW;
-    return;
-}
